import sys
sys.setrecursionlimit(10**5)
# import pdb
# import numpy as np

EXIT_OK = 0
EXIT_ERROR = 1
EXIT_OUR_ERROR = 2

def error(msg, exitcode):
    if msg == '':
        print('Wrong answer.')
    else:
        print('Wrong answer:',msg)
    exit(exitcode)

def read_token(lst, who):
    if len(lst) == 0: error("Too few tokens in the output file.", who)
    t = lst[0]
    del lst[0]
    return t

def read_map(lst, R, C, who):
    m = []
    for r in range(R):
        t = read_token(lst, who)
        if len(t) != C: error("Incorrect dimensions of a configuration of mines.", who)
        if t.count('*') + t.count('.') != C: error("An invalid character in a configuration of mines.", who)
        m.append(t)
    return m

def convert_to_int(t, who):
    try:
        t = int(t)
    except ValueError:
        error("Failed to read a number.", who)
    return t

def fill(M, r, c, R, C):
    if r<0 or c<0 or r>R-1 or c>C-1: return
    if M[r][c] != '.': return
    M[r][c] = '0'
    for dr in [-1,0,1]:
        for dc in [-1,0,1]:
            if max(abs(dr),abs(dc)) == 0: continue
            if r+dr<0 or r+dr>R-1 or c+dc<0 or c+dc>C-1: continue
            if M[r+dr][c+dc] == '*': M[r][c] = chr( ord(M[r][c]) + 1 )
    if M[r][c] != '0': return
    for dr in [-1,0,1]:
        for dc in [-1,0,1]:
            if max(abs(dr),abs(dc)) == 0: continue
            if r+dr<0 or r+dr>R-1 or c+dc<0 or c+dc>C-1: continue
            if M[r+dr][c+dc] == '.': fill(M, r+dr, c+dc, R, C)

assert len(sys.argv)==4, "correct usage: python3 %s input our_output contestants_output" % sys.argv[0]

input_tokens = open(sys.argv[1], 'r').read().split('\n\n')
our_output_tokens = open(sys.argv[2],'r').read().split()
try:
    their_output_tokens = open(sys.argv[3],'r').read().split()
except:
    error("Failed to parse the submission (binary file?)", EXIT_ERROR)

T = int(input_tokens[0])
input_tokens = input_tokens[1:]

assert( len(input_tokens) == T )

for t in range(T):
    R, C, K, ar, ac = map(int, input_tokens[t].split(' '))

    solvable = read_token(our_output_tokens, EXIT_OUR_ERROR)
    if solvable != "IMPOSSIBLE":
        solvable = convert_to_int(solvable, EXIT_OUR_ERROR)
        # if solvable >= R*C: print('Tooooooo many boards!', R, C, K, ar, ac) #error("Too many boards!", EXIT_OUR_ERROR)
        for _ in range(solvable):
            m = read_map(our_output_tokens, R, C, EXIT_OUR_ERROR)

    tok = read_token(their_output_tokens, EXIT_ERROR)
    if (solvable == "IMPOSSIBLE" or tok == "IMPOSSIBLE") and (solvable != tok):
        error("Incorrectly reported whether a solution exists.", EXIT_ERROR)

    if tok != "IMPOSSIBLE":
        tok = convert_to_int(tok, EXIT_ERROR)
        if tok > R*C: error("You cannot output more than R*C configurations of mines.", EXIT_ERROR)
        if tok <= 0: error("The number of configurations of mines must be positive.", EXIT_ERROR)
        land = [1] * (C*R)
        lor  = [0] * (C*R)
        anot = None
        for _ in range(tok):
            M = read_map(their_output_tokens, R, C, EXIT_ERROR)
            if sum( m.count('*') for m in M ) != K: error("Wrong number of mines in a configuration.", EXIT_ERROR)
            bits = [ int(b) for b in ''.join(M).replace('*','1').replace('.','0') ]
            land = [ a and b for a,b in zip(land, bits) ]
            lor  = [ a or  b for a,b in zip(lor,  bits) ]
            M = [ list(m) for m in M ]
            fill(M, ar, ac, R, C)
            if M[ar][ac] != '0': error("The revealed cell must contain a zero.", EXIT_ERROR)
            M = (''.join([''.join(m) for m in M])).replace('*','.')
            if anot is None:
                anot = M
                lst = [ (1 if c>='0' and c<='9' else 0) for c in M ]
                land = [ a and (1-b) for a,b in zip(land, lst) ]
                lor  = [ a or b for a,b in zip(lor, lst) ]
            else:
                if anot != M: error("All configurations of mines must look the same to the player.", EXIT_ERROR)
        if sum(land) != 0: error("A cell contains a mine in all the given configurations.", EXIT_ERROR)
        if sum(lor)  != len(lor): error("An unrevealed cell is empty in all the given configurations.", EXIT_ERROR)

print("OK")
exit(EXIT_OK)
